/* Copyright (c) 2005, Dmitry Xmelkov
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
   * Neither the name of the copyright holders nor the names of
     contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. */

/* $Id$ */

#include "macros.inc"
#include "xtoa_fast.h"

/* --------------------------------------------------------------------
   char * ltoa_fast (long val, char * str, int base)
   char * ultoa_fast (unsigned long val, char * str, int base)
   char * ltoa_width (long val, char * str, int base, size_t width)
   char * ultoa_width (unsigned long val, char * str, int base, size_t width)
*/
#define	XTOA_MINUS	0x8000		/* 'minus' is present	*/
#define	XTOA_DECIMAL	0x4000		/* base is 10		*/
#define	XTOA_HEX	0x2000		/* base is 16		*/
#define	XTOA_WIDTH	0x1000		/* 'width' is present	*/

    /* Input	*/
#define	v_lo	r22
#define	v_hi	r23
#define	v_hlo	r24
#define	v_hhi	r25
#define	str_lo	r20
#define	str_hi	r21
#define	base	r18
#define	flags	r19	/* high byte of `base'	*/
#define	width_lo r16
#define	width_hi r17

    /* Intermediate	*/
#define	symb	r18
#define	len_lo	r26
#define	len_hi	r27

    /* Fixed	*/
#define	zero	r1

	.text
	
	.global	ltoa_width
	.type	ltoa_width, "function"
ltoa_width:
	andi	flags, (XTOA_UPPER | XTOA_PREFIX) >> 8
	ori	flags, XTOA_WIDTH >> 8
	rjmp	.L_ltoa
	.size	ltoa_width, . - ltoa_width
	
	.global	ultoa_width
	.type	ultoa_width, "function"
ultoa_width:
	andi	flags, (XTOA_UPPER | XTOA_PREFIX) >> 8
	ori	flags, XTOA_WIDTH >> 8
	rjmp	.L_ultoa
	.size	ultoa_width, . - ultoa_width

	.global	ultoa_fast
	.type	ultoa_fast, "function"
ultoa_fast:
	andi	flags, (XTOA_UPPER | XTOA_PREFIX) >> 8
	rjmp	.L_ultoa
	.size	ultoa_fast, . - ultoa_fast

	.global	ltoa_fast
	.type	ltoa_fast, "function"
ltoa_fast:
	andi	flags, (XTOA_UPPER | XTOA_PREFIX) >> 8
.L_ltoa:
	cpi	base, 10
	brne	.L_ultoa		; sign is only for decimals
  ; sign ?
	tst	v_hhi
	brpl	.L_ultoa
  ; minus
	com	v_hhi
	com	v_hlo
	com	v_hi
	neg	v_lo
	sbci	v_hi, hi8(-1)
	sbci	v_hlo, hlo8(-1)
	sbci	v_hhi, hhi8(-1)
	ori	flags, XTOA_MINUS >> 8
.L_ultoa:
  ; prologue
	X_movw	ZL, str_lo
	push	str_hi
	push	str_lo
  ; base ?
	cpi	base, 10
	brne	1f
	andi	flags, (XTOA_MINUS | XTOA_WIDTH) >> 8	; disable prefix
	ori	flags, XTOA_DECIMAL >> 8
	rjmp	.L_conv
1:	cpi	base, 16
	brne	2f
	ori	flags, XTOA_HEX >> 8
	rjmp	.L_conv
2:	cpi	base, 8
	breq	.L_conv
	cpi	base, 2
	brne	3f
	andi	flags, XTOA_WIDTH >> 8			; disable prefix
	rjmp	.L_base_2
  ; unknown base --> no conversion
3:	clr	flags
	rjmp	.L_pop_str
  ; conversion
.L_conv:
	push	flags
	XCALL	__ultoa_invert
	X_movw	ZL, r24		; end of string ('\0')
	pop	flags
  ; minus?
	sbrs	flags, ntz(XTOA_MINUS) - 8
	rjmp	.L_pop_str
	ldi	symb, '-'
	st	Z+, symb
  ; string address restore
.L_pop_str:
	pop	r24		; str_lo
	pop	r25		; str_hi
  ; length calculation
	X_movw	len_lo, ZL
	sub	len_lo, r24
	sbc	len_hi, r25
  ; is width used?
	sbrs	flags, ntz(XTOA_WIDTH) - 8
	rjmp	.L_prefix
  ; is width too short?
	cpi	width_lo, lo8(2)
	cpc	width_hi, zero
	brlo	.L_prefix	; width <2 is too short for any base
	brne	4f		; width >2 is normal
	sbrs	flags, ntz(XTOA_HEX) - 8
	rjmp	4f
	sbrc	flags, ntz(XTOA_PREFIX) - 8
	rjmp	.L_prefix	; width 2 is too short for hex
  ; length += strlen(prefix)
4:	sbrs	flags, ntz(XTOA_PREFIX) - 8
	rjmp	5f
	adiw	len_lo, 1
	sbrc	flags, ntz(XTOA_HEX) - 8
	adiw	len_lo, 1	; prefix for hex is 2 bytes long
  ; filling
5:	ldi	symb, '0'
	sbrc	flags, ntz(XTOA_DECIMAL) - 8
	ldi	symb, ' '
6:	cp	len_lo, width_lo
	cpc	len_hi, width_hi
	brsh	.L_prefix
	st	Z+, symb
	adiw	len_lo, 1
	rjmp	6b
  ; prefix?
.L_prefix:
	sbrs	flags, ntz(XTOA_PREFIX) - 8
	rjmp	.L_reverse	; not needed
  ; val == 0 ?
	cpi	len_lo, lo8(2)		; Length checking is needed
	cpc	len_hi, zero		;   in the case of `width'.
	brsh	7f
	ld	symb, -Z
	adiw	ZL, 1
	cpi	symb, '0'
	breq	.L_reverse	; no prefix for '0'
  ; prefix output
7:	sbrs	flags, ntz(XTOA_HEX) - 8
	rjmp	8f
	ldi	symb, 'x'
	sbrc	flags, ntz(XTOA_UPPER) - 8
	ldi	symb, 'X'
	st	Z+, symb
8:	ldi	symb, '0'
	st	Z+, symb
  ; reverse string
.L_reverse:
	st	Z, zero
	XJMP	strrev

  ; base == 2
.L_base_2:
  ; lsr `val'
	lsr	v_hhi
	ror	v_hlo
	ror	v_hi
	ror	v_lo
  ; output '0' + carry
	ldi	symb, '0'
	adc	symb, zero
	st	Z+, symb
  ; tst `val'
	adiw	v_hlo, 0	; only Z is needed
	cpc	v_hi, zero
	cpc	v_lo, zero
	brne	.L_base_2
  ; epilogue
	rjmp	.L_pop_str

	.size	ltoa_fast, . - ltoa_fast	
	.end
