/* utoa_fast() test with table of patterns.  Basis are: 8,10,16.
   Also utoa_width() without expanding is tested.	*/
#include <stdlib.h>
#include <string.h>
#include "xtoa_fast.h"

/* Compare with pattern.	*/
static int cmp (unsigned int val, int base, const char * pattern)
{
    char s[8];
    char *p;
    
    p = utoa_fast (val, s, base);
    if (p != s) return 1;
    if (strcmp(s, pattern)) return 1;
    
    p = utoa_width (val, s, base, 0);
    if (p != s) return 1;
    if (strcmp(s, pattern)) return 1;
    
    return 0;
}

int main ()
{
    static struct t_s {
	unsigned int val;
	int base;
	char s[8];		/* pattern	*/
    } t[] = {

	{ 0, 10,	"0" },
	{ 0, 8,		"0" },
	{ 0, 16,	"0" },
	
	{ 1, 10,	"1" },
	{ 1, 8,		"1" },
	{ 1, 16,	"1" },
	
	{ 0xffff,	10, "65535" },
	{ 0xffff,	8, "177777" },
	{ 0xffff,	16,  "ffff" },
	
	/* Nonsensed flags must ignored.	*/
	{ 20, 10 | ~0xff,	"20" },
	{ -1, 10 | ~0xff,	"65535" },
	{ 20, 8 | (~0xff ^ XTOA_PREFIX),	"24" },
	{ -1, 8 | (~0xff ^ XTOA_PREFIX),	"177777" },
	{ 20, 16 | (~0xff ^ (XTOA_PREFIX | XTOA_UPPER)),	"14" },
	{ -1, 16 | (~0xff ^ (XTOA_PREFIX | XTOA_UPPER)),	"ffff" },
	
	/* prefix for octal	*/
	{ 0, 8 | XTOA_PREFIX,		"0" },
	{ 1, 8 | XTOA_PREFIX,		"01" },
	{ 8, 8 | XTOA_PREFIX,		"010" },
	{ -1, 8 | XTOA_PREFIX,		"0177777" },
	
	/* prefix for hex	*/
	{ 0, 16 | XTOA_PREFIX,		"0" },
	{ 1, 16 | XTOA_PREFIX,		"0x1" },
	{ 16, 16 | XTOA_PREFIX,		"0x10" },
	{ -1, 16 | XTOA_PREFIX,		"0xffff" },
	
	/* upper for hex	*/
	{ 0,      16 | XTOA_UPPER | XTOA_PREFIX,	"0" },
	{ 0xabcd, 16 | XTOA_UPPER,			"ABCD" },
	{ 0xabcd, 16 | XTOA_UPPER | XTOA_PREFIX,	"0XABCD" },
	{ 0x2,    16 | XTOA_UPPER | XTOA_PREFIX,	"0X2" },

    };
    struct t_s *p;
    int i;
    
    i = 0;
    for (p = t; p != t + sizeof(t)/sizeof(t[0]); p++) {
	i += 1;
	if (cmp(p->val, p->base, p->s))
	    exit(i);
    }
    
    return 0;
}
