/* itoa_fast() test with table of patterns.  Basis are: 8,10,16.
   Also itoa_width() without expanding is tested.	*/
#include <stdlib.h>
#include <string.h>
#include "xtoa_fast.h"

/* Compare with pattern.	*/
static int cmp (int val, int base, const char * pattern)
{
    char s[8];
    char *p;

    p = itoa_fast (val, s, base);
    if (p != s) return 1;
    if (strcmp(s, pattern)) return 2;

    p = itoa_width (val, s, base, 0);
    if (p != s) return 1;
    if (strcmp(s, pattern)) return 2;

    return 0;
}

int main ()
{
    static struct t_s {
	int val;
	int base;
	char s[8];		/* pattern	*/
    } t[] = {

	{ 0, 10,	"0" },
	{ 0, 8,		"0" },
	{ 0, 16,	"0" },
	
	{ 1, 10,	"1" },
	{ 1, 8,		"1" },
	{ 1, 16,	"1" },
	
	{ 0x7fff,	10, "32767" },
	{ 0x7fff,	8,  "77777" },
	{ 0x7fff,	16,  "7fff" },
	
	{ -1, 10,	"-1" },
	{ -1, 8,	"177777" },
	{ -1, 16,	"ffff" },
	
	{ 0x8000,	10, "-32768" },
	{ 0x8000,	8,  "100000" },
	{ 0x8000,	16,   "8000" },

	{ 0x8001,	10, "-32767" },
	{ 0x8001,	8,  "100001" },
	{ 0x8001,	16,   "8001" },

	/* Nonsensed flags must ignored.	*/
	{ 20, 10 | ~0xff,	"20" },
	{ -2, 10 | ~0xff,	"-2" },
	{ 20, 8 | (~0xff ^ XTOA_PREFIX),	"24" },
	{ -2, 8 | (~0xff ^ XTOA_PREFIX),	"177776" },
	{ 20, 16 | (~0xff ^ (XTOA_PREFIX | XTOA_UPPER)),	"14" },
	{ -2, 16 | (~0xff ^ (XTOA_PREFIX | XTOA_UPPER)),	"fffe" },
	
	/* prefix for octal	*/
	{ 0, 8 | XTOA_PREFIX,		"0" },
	{ 1, 8 | XTOA_PREFIX,		"01" },
	{ -1, 8 | XTOA_PREFIX,		"0177777" },
	
	/* prefix for hex	*/
	{ 0, 16 | XTOA_PREFIX,		"0" },
	{ 1, 16 | XTOA_PREFIX,		"0x1" },
	{ -1, 16 | XTOA_PREFIX,		"0xffff" },
	
	/* upper for hex	*/
	{ 0xabcd, 16 | XTOA_UPPER,			"ABCD" },
	{ 0xabcd, 16 | XTOA_UPPER | XTOA_PREFIX,	"0XABCD" },
	{ 0x2,    16 | XTOA_UPPER | XTOA_PREFIX,	"0X2" },

    };
    struct t_s *p;
    int i;
    
    i = 0;
    for (p = t; p != t + sizeof(t)/sizeof(t[0]); p++) {
	i += 1;
	if (cmp(p->val, p->base, p->s))
	    exit(i);
    }
    
    return 0;
}
